--[[
    Helper functions
    $Revision: 77539 $
]]--

--[[
Copyright (c) 2008, LordFarlander
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

    * Redistributions of source code must retain the above copyright notice,
      this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright notice,
      this list of conditions and the following disclaimer in the documentation
      and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
]]--

local MAJOR_VERSION = "LibLordFarlander-2.0";
local MINOR_VERSION = "$Rev: 77539 $";

local LibLordFarlander, oldminor = LibStub:NewLibrary( MAJOR_VERSION, MINOR_VERSION );
if( not LibLordFarlander ) then
    return;
end--if

local FlyableZones = nil;

-- Notes:
-- Checks if an item is soulbound or quest
-- Arguments:
--     number - bag the item is in
--    number - slot the item is in
-- Returns:
-- * boolean - if the item given is soulbound
function LibLordFarlander.ItemIsSoulbound( bag, slot )
    local gratuity = LibStub( "LibGratuity-3.0", true );

    if( gratuity ) then
        local strings;
    
        gratuity:SetBagItem( bag, slot );
        strings = gratuity:GetText( 2, 4, false, true );
        if( strings ) then
            for _, text in pairs( strings ) do
                text = text[1];
                if( ( text == ITEM_SOULBOUND ) or ( text == ITEM_BIND_QUEST ) ) then
                    return true;
                end--if
            end--for
        end--if
    end--if
    return false;
end--LibLordFarlander.ItemIsSoulbound( bag, slot )

function LibLordFarlander.PlayerItemCount( item )
    local _, itemLink, _, _, _, _, _, _, _, _ = GetItemInfo( item );

    if( itemLink ~= nil ) then
        return GetItemCount( itemLink );
    end--if
    return 0;
end--LibLordFarlander.PlayerItemCount( item )

-- Notes:
-- Gets an item string from a link
-- Arguments:
--     string - a link, can be any type of link
-- Returns:
-- * string - the item string from the link
function LibLordFarlander.GetItemStringFromLink( itemLink )
    if( itemLink ) then
        local _, _, itemString = itemLink:find( "^|c%x+|H(.+)|h%[.+%]" );
    
        return itemString;
    end--if
    return nil;
end--LibLordFarlander.GetItemStringFromLink( itemLink )

-- Notes:
-- Determines if the player is in a zone where flying is allowed
-- Returns:
-- * boolean - whether or not the player is in a zone where flying is allowed
function LibLordFarlander.IsInFlyableZone()
    if( not FlyableZones ) then
        FlyableZones = table.concat( { GetMapZones( 3 ) }, "|" );
    end--if
    return FlyableZones:find( GetRealZoneText() ) ~= nil;
end--LibLordFarlander.IsInFlyableZone()

-- Notes:
-- Gets an ID from a string
-- Arguments:
--     string - a string
-- Returns:
-- * number - the ID from the string
function LibLordFarlander.GetItemIDFromString( itemID )
    if( itemID ) then
        return tonumber( itemID:match( "(%d+)" ) );
    end--if
    return nil;
end--LibLordFarlander.GetItemIDFromString( itemID )

-- Notes:
-- Gets an ID from a link
-- Arguments:
--     string - a link, can be any type
-- Returns:
-- * number - the ID from the link
function LibLordFarlander.GetItemIDFromLink( itemLink )
    if( itemLink ) then
        return LibLordFarlander.GetItemIDFromString( LibLordFarlander.GetItemStringFromLink( itemLink ) );
    end--if
    return nil;
end--LibLordFarlander.GetItemIDFromLink( itemLink )

-- Notes:
-- Gets an item string from an ID
-- Arguments:
--     number - an item ID
-- Returns:
-- * string - the string from the ID
function LibLordFarlander.GetItemString( itemID )
    return LibLordFarlander.GetItemStringFromLink( LibLordFarlander.GetItemLink( itemID ) );
end--LibLordFarlander.GetItemString( itemID )

-- Notes:
-- Gets an item link from and ID
-- Arguments:
--     number - an item ID
-- Returns:
-- * string - the link from the ID
function LibLordFarlander.GetItemLink( itemID )
    local _, itemLink, _, _, _, _, _, _, _, _ = GetItemInfo( itemID );

    return itemLink;
end--LibLordFarlander.GetItemLink( itemID )

-- Notes:
-- Checks to see if the player has a spell
-- Arguments:
--     number, string - A spell ID or spell link
-- Returns:
-- * boolean - whether or not the player has the spell
function LibLordFarlander.HasPlayerSpell( spell )
    local spellName, _, _, _, _, _, _, _, _ = GetSpellInfo( spell );

    if( spellName ) then
        return GetSpellLink( spell ) == GetSpellLink( spellName );
    end
    return false;
end--LibLordFarlander.PlayerHasSpell( spell )

-- Notes:
-- Deep copies a table to another table
-- Arguments:
--     table - Destination table (MUST already be a table)
--     table - Source table
function LibLordFarlander.CopyTable( to, from )
    for i, v in pairs( from ) do
        if( type( v ) == "table" ) then
            to[i] = {};
            LibLordFarlander.CopyTable( to[i], v );
        else
            to[i] = v;
        end--if
    end--for
end--LibLordFarlander.CopyTable( to, from )

function LibLordFarlander.AppendTable( to, from )
    for _, v in pairs( from ) do
        to.insert( v );
    end--for
end--LibLordFarlander.AppendTable( to, from )

-- #nodef
function LibLordFarlander.SimpleCompareTables( table1, table2 )
    if( table1 and table2 ) then
        for i, v in pairs( table1 ) do
            if( type( v ) == "table" ) then
                if( type( table2[i] ) == "table" ) then
                    if( not LibLordFarlander.SimpleCompareTables( v, table2[i] ) ) then
                        return false;
                    end--if
                else
                    return false;
                end--if
            elseif( table2[i] ~= v ) then
                return false;
            end--if
        end--for
    else
        return false;
    end--if
    return true;
end--LibLordFarlander.SimpleCompareTables( table1, table2 )

-- Notes:
-- Compares two tables to see if they have all the same members and the values are the same
-- Arguments:
--     table - First table to compare
--     table - Second table to compare
-- Returns:
-- * boolean - whether or not the tables are exactly alike
function LibLordFarlander.CompareTables( table1, table2 )
    return LibLordFarlander.SimpleCompareTables( table1, table2 ) and LibLordFarlander.SimpleCompareTables( table2, table1 );
end--LibLordFarlander.CompareTables( table1, table2 )

-- Notes:
-- Debug message printing, suitable for including in your addon's class.
-- Set your addon's DEBUG member to true to print.
-- Arguments:
--     addon - The addon object
--     string - The message to print
function LibLordFarlander.dprint( addon, msg )
    if( addon.DEBUG ) then
        DEFAULT_CHAT_FRAME:AddMessage( msg, 1.0, 0.0, 0.0 );
    end--if
end--LibLordFarlander.dprint( addon, msg )
